@@ -1,14 +1,13 @@
-from base64 import urlsafe_b64encode
 from os import urandom
 
+from base64 import urlsafe_b64encode
+from common.models import Class
 from django.contrib.auth.models import User
 from django.db import models
+from wagtail.snippets.models import register_snippet
 
 from aimmo import app_settings
 
-from common.models import Class
-
-
 DEFAULT_WORKSHEET_ID = 1
 
 GAME_GENERATORS = [("Main", "Open World")] + [  # Default
@@ -20,6 +19,12 @@ def generate_auth_token():
     return urlsafe_b64encode(urandom(16))
 
 
+class WorksheetManager(models.Manager):
+    def sorted(self):
+        return self.get_queryset().order_by("sort_order")
+
+
+@register_snippet
 class Worksheet(models.Model):
     ERA_CHOICES = [
         (1, "future"),
@@ -30,9 +35,21 @@ class Worksheet(models.Model):
     ]
 
     name = models.CharField(max_length=100)
+    description = models.TextField(blank=True)
+    short_description = models.TextField(blank=True)
+    image_path = models.CharField(max_length=255, blank=True)
+    active_image_path = models.CharField(max_length=255, blank=True)
     era = models.PositiveSmallIntegerField(
         choices=ERA_CHOICES, default=ERA_CHOICES[0][0]
     )
+    thumbnail_text = models.CharField(max_length=100, blank=True)
+    thumbnail_image_path = models.CharField(max_length=255, blank=True)
+    teacher_pdf_name = models.CharField(max_length=255, blank=True)
+    student_pdf_name = models.CharField(max_length=255, blank=True)
+    sort_order = models.IntegerField(default=0)
+
+    objects = WorksheetManager()
+
     starter_code = models.TextField()
 
     def __str__(self):
@@ -45,16 +62,21 @@ class Game(models.Model):
     PAUSED = "p"
     STATUS_CHOICES = ((RUNNING, "running"), (STOPPED, "stopped"), (PAUSED, "paused"))
 
-    name = models.CharField(max_length=100)
+    name = models.CharField(max_length=100, blank=True, null=True)
     auth_token = models.CharField(max_length=24, blank=True)
-    owner = models.ForeignKey(User, blank=True, null=True, related_name="owned_games")
-    game_class = models.ForeignKey(
+    owner = models.ForeignKey(
+        User,
+        blank=True,
+        null=True,
+        related_name="owned_games",
+        on_delete=models.SET_NULL,
+    )
+    game_class = models.OneToOneField(
         Class,
         verbose_name="Class",
         on_delete=models.CASCADE,
         blank=True,
         null=True,
-        related_name="games_for_class",
     )
     public = models.BooleanField(default=False)
     can_play = models.ManyToManyField(
@@ -64,7 +86,11 @@ class Game(models.Model):
     )
     completed = models.BooleanField(default=False)
     main_user = models.ForeignKey(
-        User, blank=True, null=True, related_name="games_for_user"
+        User,
+        blank=True,
+        null=True,
+        related_name="games_for_user",
+        on_delete=models.SET_NULL,
     )
     static_data = models.TextField(blank=True, null=True)
 
@@ -81,13 +107,9 @@ class Game(models.Model):
     start_height = models.IntegerField(default=31)
     start_width = models.IntegerField(default=31)
     status = models.CharField(max_length=1, choices=STATUS_CHOICES, default=RUNNING)
-    worksheet = models.ForeignKey(Worksheet)
-
-    class Meta:
-        unique_together = (
-            "game_class",
-            "worksheet",
-        )
+    worksheet = models.ForeignKey(
+        Worksheet, on_delete=models.PROTECT, default=DEFAULT_WORKSHEET_ID
+    )
 
     @property
     def is_active(self):
@@ -135,19 +157,10 @@ class Game(models.Model):
 
 
 class Avatar(models.Model):
-    owner = models.ForeignKey(User)
-    game = models.ForeignKey(Game)
+    owner = models.ForeignKey(User, on_delete=models.CASCADE)
+    game = models.ForeignKey(Game, on_delete=models.CASCADE)
     code = models.TextField()
     auth_token = models.CharField(max_length=24, default=generate_auth_token)
 
     class Meta:
         unique_together = ("owner", "game")
-
-
-class LevelAttempt(models.Model):
-    level_number = models.IntegerField()
-    user = models.ForeignKey(User)
-    game = models.OneToOneField(Game)
-
-    class Meta:
-        unique_together = ("level_number", "user")
